#ifndef TIMESERIES_JT
#define TIMESERIES_JT

#include "Network.h"
#include "Evolution.h"

/**
 * @class TimeSeries
 * 
 * Structure to hold multiple Networks across a number of time steps, and to run calculations of high level processes (ie evolution detection)
 */
class TimeSeries{
public:
  /**
   * Constructors
   */
  explicit TimeSeries(){};
  //explicit TimeSeries(const std::string& filename, const std::string delim){LoadCommunityData(filename, delim);};
  explicit TimeSeries(const std::string& filename, const std::string& delimiterA, const std::string& delimiterB, const double& wt, const bool positive){LoadData(filename, delimiterA, delimiterB, wt, positive);};
 
  /**Load Functions*/
  void LoadData(const std::string& filename, const std::string& delimiterA, const std::string& delimiterB, const double& wt, const bool positive);
  //void LoadCommunityData(const std::string& filename, const std::string& delimiter);
  
  /**Calculation Functions*/
  void CalculateSimilarities(const int& stepSize, const double& threshold);
  void PrintEvolutions(const int& minLength);

  void ExtendEvolutions(const int& minSize, const int& timegap, const int& stubSize, const double& threshold);
  
  /**Destructor*/
  ~TimeSeries(){};
  
private:
  std::vector < std::tr1::shared_ptr<Network> > timestep; //Structure for temporal network
  std::set < EvolutionPtr > evolutions;
  
  /** Disallow copying */
  TimeSeries& operator=(const TimeSeries&);
  TimeSeries& operator=(const TimeSeries*);
  TimeSeries(const TimeSeries&);
  TimeSeries(const TimeSeries*);
};

#endif
