#include "Timestamp.h"

Timestamp::Timestamp(const string& date):dateStr(date){
    vector < string > fields;
    string dcopy = date;
    split_tr(dcopy, &fields, " -:\n\r");

    valid = true;

    if (fields.size() < 6){
      valid = false;
      return;
    }
   
    Y = str_to<unsigned short int>(fields[0]);
    M = str_to<unsigned short int>(fields[1]);
    D = str_to<unsigned short int>(fields[2]);
    H = str_to<unsigned short int>(fields[3]);
    N = str_to<unsigned short int>(fields[4]);
    S = str_to<unsigned short int>(fields[5]);

    CheckBounds();
  };

bool Timestamp::operator< (const Timestamp& other){
  const Timestamp temp(getstr());
  return temp < other;
}

bool Timestamp::operator> (const Timestamp& other){
  const Timestamp temp(getstr());
  return temp > other;
}

bool Timestamp::operator> (const Timestamp& other) const{
  return !(*this < other) && !(*this == other);
}

bool Timestamp::operator== (const Timestamp& other){
  const Timestamp temp(getstr());
  return temp == other;
}

bool Timestamp::operator== (const Timestamp& other) const{
  return (Y == other.Y) && (M == other.M) && (D == other.D) && (H == other.H) && (N == other.N) && (S == other.S);
}

bool Timestamp::operator< (const Timestamp& other) const{
    if (Y == other.Y){
      if (M == other.M){
	if (D == other.D){
	  if (H == other.H){
	    if (N == other.N){
	      if (S == other.S){
		return false;
	      } else {
		return S < other.S;
	      }
	    } else {
	      return N < other.N;
	    }
	  } else { 
	    return H < other.H;
	  }
	} else {
	  return D < other.D;
	}
      } else {
	return M < other.M;
      }
    } else {
      return Y < other.Y;
    }
  }

Timestamp Timestamp::operator= (const Timestamp& other){
    Y = other.Y;
    M = other.M;
    D = other.D;
    H = other.H;
    N = other.N;
    S = other.S;
    
    return *this;
  }

Timestamp Timestamp::operator+ (const Timestamp& other){
  Timestamp res(getstr());
  res.Y += other.Y;
  res.H += other.H;
  res.M += other.M;
  res.D += other.D;
  res.N += other.N;
  res.S += other.S;
  
  res.Reduce();
    
  return res;
}

bool Timestamp::CheckBounds(){
    if (Y < 0) return Error("Negative Year? No.");

    if (M < 0 || M > 12) 
      return Error("Invalid month : " + to_str<unsigned short int>(M));

    int topBound;

    switch (M){
    case 4:
    case 6:
    case 9:
    case 11:
      if (D < 0 || D > 30) 
	return Error("Invalid day");
      break;

    case 2:
      topBound = 28;
      if (Y % 4 == 0)
	++topBound;

      if (D < 0 || D > topBound) 
	return Error("Invalid day");
      break;

    default:
      if (D < 0 || D > 31) 
	return Error("Invalid day");
      break;
    }

    if (H < 0 || H > 23) return Error("Invalid Hour");
    
    if (N < 0 || N > 59) return Error("Invalid Minute");

    if (S < 0 || S > 59) return Error("Invalid Seconds");

    return true;
  }

void Timestamp::Reduce(){
  //if (CheckBounds()) return;

    if (S > 59){
      N += S / 60;
      S = S % 60;
    }

    if (N > 59){
      H += N / 60;
      N = N % 60;
    }

    if (H > 23){
      D += H / 24;
      H = H % 24;
    }
    
    bool inv = true;
    while (inv){
      int topBound = 31;
      switch (M){
      case 4:
      case 6:
      case 9:
      case 11:
	topBound = 30;
	break;
      case 2:
	topBound = 28;
	if (Y % 4 == 0)
	  ++topBound;
	break;
      default:
	break;
      }

      if (D > topBound){
	++M;
	D = D - topBound;
      } else {
	inv = false;
      }
    }

    if (M > 12){
      Y += M / 12;
      M = M % 12;
    }
  }

bool Timestamp::Error(const string& message){
    cerr << message << endl;
    return false;
  }

